///////    GRubber.java    ///////

import java.awt.geom.*;
import java.awt.*;

/**
 * Clase de dibujo del elástico.
 * @author Martín Cortés, Mauricio Ludueña
 */
class GRubber extends Rubber implements Drawable {
  //Atributos privados no aparecen en el javadoc
   private static final int Points = 50;
   private static final Path2D.Double polyline = new Path2D.Double(Path2D.WIND_EVEN_ODD,Points);
   private Path2D.Double shape;
   private Stroke stroke;
	
	private Color color = Color.BLACK;
   private Vector2D freeAendPosition=null;  // to draw a spring we need
   private Vector2D freeBendPosition=null;  // extremes well definied
                                      // when it is not connected
  
  
  	
static {  
      double tam;
      polyline.moveTo (0, 0);
      for (int index = 0; index < Points;index++){
         tam =  (double)index/Points;
         polyline.lineTo(tam, 0.001*Math.sin(index*Math.PI/4));
      }
   }
	
	
        /** 
        * Constructor.          Crea un nuevo grafico de elástico.
        * @param restLenght      Largo en reposo del elástico.
        * @param stiffness      Viscosidad del elástico.
        */
      public GRubber(float restLength, float stiffness) {
      super(restLength, stiffness);
      freeAendPosition = new Vector2D(0,0);
      freeBendPosition = new Vector2D(restLength,restLength);
      AffineTransform at = AffineTransform.getTranslateInstance(0,0);
      Vector2D v = getVector();
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(), restLength*restLength/v.module());
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      stroke = new BasicStroke(1);
   }
 
        /**
        * Devuelve la posicion del extremo A del elástico.
        * @return Vector Posicion del extremo.
        */	
   public Vector2D getAendPosition() {
      if (a_end == null)
         return freeAendPosition;
      return super.getAendPosition();
   }
        /**
        * Devuelve la posicion del extremo B del elástico.
        * @return Vector Posicion del extremo.
        */
   public Vector2D getBendPosition() {
      if (b_end == null)
         return freeBendPosition;
      return super.getBendPosition();
   }
        /**
        * Dibuja el elástico.
        * @param g Objeto grafico de 2 dimensiones. .
        */
   public void draw (Graphics2D g){
      Vector2D a=getAendPosition();
      double ax = a.getX();
      double ay = a.getY();
      Vector2D v = getVector();
      AffineTransform at = AffineTransform.getTranslateInstance(ax, ay);
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(),  restLength);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      if (v.module() < restLength)
         g.setColor(Color.BLACK);
      else
         g.setColor(Color.RED);
      g.setStroke(stroke);
      g.draw(shape);
   }
        /**
        * Determina si el objeto contiene a otro.
        * @param x Coordenada X del lugar.
        * @param y Coordenada Y del lugar.
        * @return True si contiene a otro, False si no.
        */   
   public boolean contains(int x, int y){
      return shape.getBounds2D().contains(x,y);
   }
        /**
        * Arrastra el extremo del elástico.
        * @param x Coordenada X del lugar.
        * @param y Coordenada Y del lugar.
        * @return True si contiene a otro, False si no.
        */
   public void dragTo(int x, int y){
      Vector2D a=getAendPosition();
      Vector2D b=getBendPosition();
      double da = Math.abs(a.getX()-x)+Math.abs(a.getY()-y); // "distance" to a
      double db = Math.abs(b.getX()-x)+Math.abs(b.getY()-y); // "distance" to b
      if (da < db) {
         detachAend();
         freeAendPosition.set(x,y);
      }
      else {
         detachBend();
         freeBendPosition.set(x,y);
      }
   }
        /**
        * Actualiza el estado del elástico.
        */
   public void updateState(){  // just to return to a rest length
      if (a_end==null && b_end==null) return;  // when it is released
      if (a_end!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         freeBendPosition = getAendPosition().plus(v);
         return;
      }
      if (b_end!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         freeAendPosition = getBendPosition().minus(v);
         return;
      }
   }
        /**
        * Marca el elástico como seleccionado.
        */
   public void setSelected() {
      stroke = new BasicStroke(3);
   }
        /**
        * Se deja de marcar el elástico como seleccionado.
        */
   public void setReleased() {
      stroke = new BasicStroke(1);
   }
}